#include "ChrPage.h"
#include "Graphics.h"
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/wfstream.h>

ChrPage::ChrPage(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size, long style, const wxString &name)
	: EditWindow(parent, id, pos, size, style, name), chr(nullptr), length(0), bitsize(BITSIZE_2BPP)
{}

ChrPage::~ChrPage() {
	delete[] chr;
}

int ChrPage::GetTileSideLength() const {
	return 10;			//8 + 1 pixel for each side (for the boundary between tiles)
}

int ChrPage::GetMargin() const {
	return 1;
}

void ChrPage::LoadTiles() {
	wxFileDialog openfile(this, "Open CHR file", "", "", "All files (*.*)|*.*",
		wxFD_OPEN | wxFD_FILE_MUST_EXIST);
	if (openfile.ShowModal() == wxID_CANCEL) return;
	wxFileName temp(openfile.GetPath());
	length = temp.GetSize().GetLo();
	delete[] chr;
	chr = new unsigned char[length];
	wxFileInputStream stream(openfile.GetPath());
	stream.ReadAll(chr, length);
	SetSizeInTiles(0x10, length / GetBitSizeInBytes(bitsize) / 0x10
		+ (length % GetBitSizeInBytes(bitsize) > 0 ? 1 : 0));
	SetIsLoaded(true);
	Invalidate();
}

void ChrPage::SetPaletteWindowPtr(PaletteWindow *pal) {
	palWin = pal;
}

void ChrPage::SetBitSize(int size) {
	bitsize = size;
}

PaletteWindow *ChrPage::GetPaletteWindowPtr() const {
	return palWin;
}

void ChrPage::OnDrawATile(int tileindex) {
	const PointDrawType lambda_pal = [&](int x, int y, int index) {
		if (bitsize == BITSIZE_3BPP_RIGHT) index += 8;
		DrawPixel(x, y, palWin->GetRgbColor(index));
	};
	const PointDrawType lambda_dir = [&](int x, int y, int color) {
		DrawPixel(x, y, SfcColorToRGB(color));
	};

	switch (bitsize) {
	case BITSIZE_2BPP:
		DrawTile_2bpp(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_pal);
		break;
	case BITSIZE_3BPP_LEFT:
	case BITSIZE_3BPP_RIGHT:
		DrawTile_3bpp(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_pal);
		break;
	case BITSIZE_4BPP:
		DrawTile_4bpp(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_pal);
		break;
	case BITSIZE_8BPP:
		DrawTile_8bpp(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_pal);
		break;
	case BITSIZE_MODE_7:
		DrawTile_Mode7(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_pal);
		break;
	case BITSIZE_DIR:
		DrawTile_DirectColor(chr + tileindex * GetBitSizeInBytes(bitsize), lambda_dir);
		break;
	}
}
