#ifndef EDITWINDOW_H__
#define EDITWINDOW_H__

#include <wx/scrolwin.h>

class EditWindow : public wxScrolled<wxWindow> {
public:
	EditWindow() = delete;
	EditWindow(wxWindow *parent, wxWindowID id = -1, const wxPoint &pos = wxDefaultPosition,
		const wxSize &size = wxDefaultSize, long style = wxHSCROLL | wxVSCROLL, 
		const wxString &name = "editWindow");
	virtual ~EditWindow();

	virtual int GetTileSideLength() const;

	void SetSizeInTiles(int xtiles, int ytiles);

	int GetWidthInTiles() const;

	int GetHeightInTiles() const;

	void SetZoom(int zoomLevel);

	int GetZoom() const;

	//factors out zoom
	wxPoint GetTileDisplayStart() const;

	//factors out zoom
	wxSize GetTileDisplaySize() const;

	//factors out zoom
	wxRect GetTileDisplayArea() const;

	//will not go beyond bounds of visual client area
	wxRect GetClippedTileRectangle(int xpos, int ypos) const;

	//will not go beyond bounds of visual client area
	wxRect GetClippedTileRectangle(wxPoint pt) const;
	
	//may go beyond bounds of visual client area
	wxRect GetFullTileRectangle(int xpos, int ypos) const;

	//may go beyond bounds of visual client area
	wxRect GetFullTileRectangle(wxPoint pt) const;

	//gets index
	int GetIndexFromPoint(int xpos, int ypos) const;

	//gets index
	int GetIndexFromPoint(wxPoint pt) const;

	//do not use
	//reserved for wxScrolled<wxWindow> superclass
	virtual void OnDraw(wxDC& dc);

	//invalidates parent
	void Invalidate();

	bool GetIsLoaded() const;

protected:
	void SetSizeInTiles();

	void SetIsLoaded(bool isSo);

	//only works within OnDrawATile() and functions called from it
	void DrawPixel(int x, int y, const wxColour& color);

	//only works within OnDrawATile() and functions called from it
	//draws based on currentClippedRectangle
	void DrawRect(const wxColour& color);

private:
	//override in derived class for drawing each tile
	virtual void OnDrawATile(int tileindex) = 0;

	bool FallsWithinClippedRectangle(const wxPoint& pt, const wxRect& clipped) const;

	virtual int GetMargin() const;

	int widthInTiles;
	int heightInTiles;
	int zoom;
	bool isLoaded;
	wxDC *currentDC;
	wxRect currentClippedRect;
	wxRect currentFullRect;
	int currentXScroll;
	int currentYScroll;
	
};

#endif
