//SFC graphics functions (header)
//by beneficii

#ifndef GRAPHICS_H__
#define GRAPHICS_H__

#include <functional>
#include <wx/colour.h>

#define BITSIZE_2BPP		0
#define BITSIZE_3BPP_LEFT	1
#define BITSIZE_3BPP_RIGHT	2
#define BITSIZE_4BPP		3
#define BITSIZE_8BPP		4
#define BITSIZE_MODE_7		6
#define BITSIZE_DIR			7

//gets the number bytes per tile, per bitsize index
std::size_t GetBitSizeInBytes(int bitsize);

//convert SFC color to 24-bit RGB color
void SfcColorToRGB(int color, int *red, int *green, int *blue);

wxColour SfcColorToRGB(int color);

//convert 24-bit RGB color to SFC color
int RGBToSfcColor(int red, int green, int blue);

//convert colors from one buffer to another
void AllRGBToSfcColor(unsigned char *write, const unsigned char *read, int num_colors);

//this function must take an int of x and of y, and must draw them to the origin using the color coded in the 3rd int
//so int x, int y, int color/color
//the color/index represents the palette index or (in direct color mode) the snes color
typedef std::function<void(int, int, int)> PointDrawType;

//need 16 bytes
void DrawTile_2bpp(const unsigned char *tiledata, PointDrawType function);

//need 24 bytes
void DrawTile_3bpp(const unsigned char *tiledata, PointDrawType function);

//need 32 bytes
void DrawTile_4bpp(const unsigned char *tiledata, PointDrawType function);

//need 64 bytes
void DrawTile_8bpp(const unsigned char *tiledata, PointDrawType function);

//need 64 bytes
void DrawTile_Mode7(const unsigned char *tiledata, PointDrawType function);

//need 128 bytes
void DrawTile_DirectColor(const unsigned char *tiledata, PointDrawType function);

//for each tile in the tilemap
struct TileMap_Tile {
	bool verticalFlip;
	bool horizontalFlip;
	bool priorityBit;
	int paletteNumber;
	int tileNumber;
};

TileMap_Tile GetTileMap_Tile(int data);
int GetTileMap_TileData(const TileMap_Tile &tile);

#endif
