#include "PalWindow.h"
#include <wx/filedlg.h>
#include <wx/wfstream.h>
#include <wx/brush.h>
#include <wx/dc.h>
#include <cstring>
#include "Graphics.h"

static constexpr void MemSet_(unsigned char *ptr, unsigned char val, std::size_t size) {
	for (std::size_t i = 0; i < size; ++i)
		*ptr++ = val;
}

PaletteWindow::PaletteWindow(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size, long style, const wxString &name)
	: EditWindow(parent, id, pos, size, style, name) {
	MemSet_(palette, 0, 0x200);
	SetSizeInTiles(0x10, 0x10);
}

int PaletteWindow::GetTileSideLength() const {
	return 16;
}

wxColour PaletteWindow::GetRgbColor(int index) {
	index &= 0xff;
	int colortemp = (int)palette[index * 2];
	colortemp |= (int)(palette[1 + index * 2] << 8);
	return SfcColorToRGB(colortemp);
}

wxColour PaletteWindow::GetRgbColor(int x, int y) {
	return GetRgbColor(GetIndexFromPoint(x, y));
}

void PaletteWindow::Load24BitRGBPalette() {
	wxFileDialog openfile(this, "Open 24-bit RGB Palette file", "", "", "All files (*.*)|*.*",
		wxFD_OPEN | wxFD_FILE_MUST_EXIST);
	if (openfile.ShowModal() == wxID_CANCEL) return;
	unsigned char input[0x300];
	std::memset(input, 0, 0x300);
	wxFileInputStream stream(openfile.GetPath());
	stream.ReadAll(input, 0x300);						// only read the first 0x300
	AllRGBToSfcColor(palette, input, 0x100);
	SetIsLoaded(true);
	Invalidate();
}

void PaletteWindow::LoadPalette() {
	wxFileDialog openfile(this, "Open SFC Palette file", "", "", "All files (*.*)|*.*",
		wxFD_OPEN | wxFD_FILE_MUST_EXIST);
	if (openfile.ShowModal() == wxID_CANCEL) return;
	wxFileInputStream stream(openfile.GetPath());
	stream.ReadAll(palette, 0x200);
	SetIsLoaded(true);
	Invalidate();
}

void PaletteWindow::OnDrawATile(int tileindex) {
	DrawRect(GetRgbColor(tileindex));
}
